<?php

namespace MediaTranscription\MediaTranscription\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use MediaTranscription\MediaTranscription\Forms\Transcription as TranscriptionForm;
use MediaTranscription\MediaTranscription\Http\Requests\TranscriptionRequest;
use MediaTranscription\MediaTranscription\Http\Resources\TranscriptionResource;
use MediaTranscription\MediaTranscription\Http\Resources\TranscriptionsCollection;
use MediaTranscription\MediaTranscription\Models\Transcription;
use MediaTranscription\MediaTranscription\Scopes\TranscriptionResourceScope;


/**
 * Resource controller class for transcription.
 */
class TranscriptionResourceController extends BaseController
{

    /**
     * Initialize transcription resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = TranscriptionForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('media_transcription.media_transcription.modules'), 'media_transcription', guard_url('media_transcription'));
    }

    /**
     * Display a list of transcription.
     *
     * @return Response
     */
    public function index(TranscriptionRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Transcription::pushScope(new RequestScope())
            ->pushScope(new TranscriptionResourceScope())
            ->simplePaginate($pageLimit);

        $data = new TranscriptionsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('media_transcription::transcription.names'))
            ->view('media_transcription::transcription.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display transcription.
     *
     * @param Request $request
     * @param Model   $transcription
     *
     * @return Response
     */
    public function show(TranscriptionRequest $request, Transcription $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TranscriptionResource($model);

        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('media_transcription::transcription.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('media_transcription::transcription.show')
            ->output();

    }

    /**
     * Show the form for creating a new transcription.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TranscriptionRequest $request, Transcription $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TranscriptionResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('media_transcription::transcription.name'))
            ->view('media_transcription::transcription.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new transcription.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TranscriptionRequest $request, Transcription $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new TranscriptionResource($model);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('media_transcription::transcription.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('media_transcription/transcription/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/media_transcription/transcription'))
                ->redirect();
        }

    }

    /**
     * Show transcription for editing.
     *
     * @param Request $request
     * @param Model   $transcription
     *
     * @return Response
     */
    public function edit(TranscriptionRequest $request, Transcription $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TranscriptionResource($model);

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('media_transcription::transcription.name'))
            ->view('media_transcription::transcription.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the transcription.
     *
     * @param Request $request
     * @param Model   $transcription
     *
     * @return Response
     */
    public function update(TranscriptionRequest $request, Transcription $model)
    {
        try {
            $attributes = $request->all();
            $model = $model->update($attributes);
            $data = new TranscriptionResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('media_transcription::transcription.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('media_transcription/transcription/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('media_transcription/transcription/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the transcription.
     *
     * @param Model   $transcription
     *
     * @return Response
     */
    public function destroy(TranscriptionRequest $request, Transcription $model)
    {
        try {
            $model->delete();
            $data = new TranscriptionResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('media_transcription::transcription.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('media_transcription/transcription/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('media_transcription/transcription/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
